package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.service.HtAnalogDataService;
import com.bizmatics.service.impl.HtAnalogDataServiceImpl;
import com.bizmatics.service.vo.CommonIcoVO;
import com.bizmatics.service.vo.HadCountVO;
import com.bizmatics.service.vo.TimeShareVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;

/**
 * 历史消费组模拟量
 *
 * @author ya
 * @since 2021-07-07
 */
@RestController
@RequestMapping("/htAnalogData")
public class HtAnalogDataController {

    @Autowired
    private HtAnalogDataService htAnalogDataService;

    /**
     * 首页日/月/年用电量
     * @return
     */
    @GetMapping("count")
    public ApiResult<HadCountVO> count(){
        return ApiResult.success(htAnalogDataService.selectCount());
    }

    /**
     * 今日/昨日用电趋势
     * @param siteId 站点id
     * @return
     */
    @GetMapping("eptrendIco")
    public ApiResult<List<CommonIcoVO>> getTrendByDate(@RequestParam(required = false) Integer siteId){
        return ApiResult.success(htAnalogDataService.selectTrendByDate(siteId));
    }

    /**
     * 当月用电趋势图
     * @return
     */
    @GetMapping("eptrendIcoMonth")
    public ApiResult<List<CommonIcoVO>> getTrendByMonth(){
        return ApiResult.success(htAnalogDataService.selectTrendByMonth());
    }


    /**
     * 站点日/月/年用电量
     * @param siteId 站点id
     * @return
     */
    @GetMapping("dayMonthYearEp")
    public ApiResult<HadCountVO> getCountBySite(@RequestParam Integer siteId){
        return ApiResult.success(htAnalogDataService.getCountBySite(siteId));
    }


    /**
     * 分时电量
     * @param siteId 站点id
     * @param type 时间
     * @return
     */
    @GetMapping("timeShare")
    public ApiResult<TimeShareVO> getTimeShare(@RequestParam Integer siteId,@RequestParam Integer type){
        return ApiResult.success(htAnalogDataService.getTimeShare(siteId,type));
    }

    /**
     * 需量趋势图
     * @param siteId 站点id
     * @return
     */
    @GetMapping("demandIco")
    public ApiResult<List<CommonIcoVO>> getDemandIco(@RequestParam Integer siteId){
        return ApiResult.success(htAnalogDataService.getDemandIco(siteId));
    }

    /**
     * 历史记录电流电压
     * @param siteId 站点id
     * @param date 时间
     * @return
     */
    @GetMapping("electricIco")
    public ApiResult<List<CommonIcoVO>> getElectricIco(@RequestParam Integer siteId,@RequestParam Date date){
        return ApiResult.success(htAnalogDataService.getElectricIco(siteId,date));
    }
}

