package com.bizmatics.service.util;

import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.model.User;
import lombok.extern.log4j.Log4j2;

import java.util.Optional;

import static com.bizmatics.common.core.exception.BusinessErrorCode.BIZ_LACK_NECESSARY_PARAM_ERROR;

/**
 * @author yq
 * @date 2021/7/8 10:34
 */
@Log4j2
public class SessionLocal {
    private static ThreadLocal<User> local = new ThreadLocal<User>();

    /**
     * 设置用户信息
     *
     * @param user
     */
    public static void setUser( User user )
    {
        local.set( user );
    }

    /**
     * 获取登录用户信息
     *
     * @return
     */
    public static User getUser() {
        log.info( "当前线程：" + Thread.currentThread().getName() );
        return local.get();
    }

    public static Integer getUserId(){
        return Optional.ofNullable(getUser()).map(User::getId).orElseThrow(() ->new BusinessException(BIZ_LACK_NECESSARY_PARAM_ERROR,"用户未登录"));
    }
}
